#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#define LOGFILE "listen.log"
#define PACK_PREFIX "pack"
#define PACK_DIR "."
#define LOGDIR "."
#define BUFSIZE 256

typedef struct sockaddr_in SA;

unsigned int seq=0;

void savepack(SA *,int,char *,int,unsigned int);

void usage(char *myname)
{
	fprintf(stderr,"Usage: %s portnum\n",myname);
	exit(1);
}

int main(int argc, char **argv)
{
 char buf[BUFSIZE+1];
 time_t t;
 int sockfd,port,len,packlen;
 unsigned int flags;
 struct sockaddr_in srvaddr,cltaddr;
 
 if (argc!=2) usage(argv[0]);
 if (!(port=atoi(argv[1]))) usage(argv[2]); 
 
#ifndef DEBUG
  snprintf(buf,BUFSIZE,"%s/%s",LOGDIR,LOGFILE);
  if (access(LOGDIR,W_OK)&&access(buf,W_OK))
   	{ fprintf(stderr,"Can not write to logfile:%s\nor logdir %s is not"
	        " writable.\n",buf,LOGDIR);
		return -1;
		}
#endif
  if(access(PACK_DIR,W_OK)) {
   	fprintf(stderr,"packets saving directory is not writable\n");
	return -1;
	}

#ifndef DEBUG
if (fork()) return 0; /* father dies. child goes background */
	 close(0);
	 close(1);
	 close(2);
	 setsid();
	 umask(0);
	 open(buf,O_WRONLY|O_CREAT|O_APPEND,0755);
	 dup(0);
	 dup(0); /* all logs and stdout is the same */
#endif
	 time(&t);
	 /* point the time when started */
         printf("-=%s",asctime(localtime(&t)));
	 fflush(stdout);
	 if((sockfd = socket(AF_INET,SOCK_DGRAM,0))==-1) {
		 perror("socket:");
		 return 1;
	 }
	 bzero(&srvaddr,sizeof(srvaddr));
	 srvaddr.sin_family=AF_INET;
	 srvaddr.sin_addr.s_addr=htonl(INADDR_ANY);
	 srvaddr.sin_port=htons(port);

	 if(bind(sockfd,(struct sockaddr *) &srvaddr,sizeof(srvaddr))) {
		perror("bind:");
		return 1;
	 }

	 /* start loggin' packets here */
	 for( ; ; ) {
		 len=sizeof(cltaddr);
		 if((packlen=recvfrom(sockfd,buf,BUFSIZE,flags,
			&cltaddr,&len))==-1) {
			 			perror("recvfrom:");
						fflush(stderr);
						return 0;
		 }
#ifdef PONG
		 sendto(sockfd,buf,packlen,0,&cltaddr,len);/*pong*/
#endif
		 savepack(&cltaddr,len,buf,packlen,flags);
	 }/* never reached */
	 return 0;
}


void savepack(SA *cltaddr,int addrlen,char *buf,int buflen,unsigned int flags) {
	char packfile[BUFSIZE+1];
	time_t t;
	int fd;

	time(&t);
	printf("[0x%08X]UDP-->%s:%i(%i)\t[0x%04X]\t%s",seq,
		inet_ntoa(cltaddr->sin_addr),
		ntohs(cltaddr->sin_port),buflen,flags,asctime(localtime(&t)));
	seq++;/* we just need to count sequences somehow */
	fflush(stdout);
	snprintf(packfile,BUFSIZE,"%s/%s.%s.%i.%i.0x%08X",PACK_DIR,PACK_PREFIX,
			inet_ntoa(cltaddr->sin_addr),ntohs(cltaddr->sin_port),
			(unsigned int)t,seq);
	if(!access(packfile,F_OK)) {
		printf("file %s exist.(no overwrite)\n",packfile);
		fflush(stdout);
		return;/* we dont want to overwrite existing file */
	}
	if((fd=open(packfile,O_WRONLY|O_CREAT))==-1) {
		perror("can not open pack.file:");
		fflush(stderr);
		return;
	}
	if (write(fd,buf,buflen)!=buflen) {
		perror("write error:");
		fflush(stderr);
	}
	close(fd);
	return;
}
	
